function result = andsegments(varargin);

%ANDSEGMENTS Determine intersection of segment lists
%
% ANDSEGMENTS returns the segment list corresponding to the intersection of
% the individual segment lists provided as arguments.  ANDSEGMENTS allows
% an arbitrary number of input segment lists.
%
%SYNOPSIS
%   LIST = andsegments(LIST1, LIST2, ...);
%
%INPUTS
%   LIST1, LIST2, ... - Segments lists in the form of N-by-2 matrices,
%   formed from adjacent column-wise vectors of segment start and stop times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% As an alternative, each segment may also be specified by its start time
% and duration.  However, the resulting segment list will be returned in
% terms of start and stop times.
%
%OUTPUT
%   LIST - intersecation of input lists
%
% ANDSEGMENTS also uses CULLSEGMENTS and MERGESEGMENTS to remove zero
% length segments and merge any overlapping or adjacent segments.
%
% See also NANDSEGMENTS, ORSEGMENTS, and NORSEGMENTS.
%
%AUTHOR
%   Shourov K. Chatterji (No longer part of the LV Collaboration.)

% $Id: andsegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(1, Inf, nargin));

% number of segment lists
lists = nargin;

% standardize segment lists
for list = 1 : lists,
  varargin{list} = stdsegments(varargin{list});
end

% remove zero length segments from lists
for list = 1 : lists,
  varargin{list} = cullsegments(varargin{list});
end

% merge overlapping segments within each list
for list = 1 : lists,
  varargin{list} = mergesegments(varargin{list});
end

% handle empty lists
for list = 1 : lists,
  if isempty(varargin{list}),
    result = [];
    return;
  end
end

% extract start and stop vectors
start = cell(1, lists);
stop = cell(1, lists);
for list = 1 : lists,
  start{list} = varargin{list}(:,1);
  stop{list} = varargin{list}(:,2);
end

% combine segment lists
times = [];
edges = [];
for list = 1 : lists,
  times = [times; start{list};];
  edges = [edges; ones(size(start{list}));];
end
for list = 1 : lists,
  times = [times; stop{list};];
  edges = [edges; -ones(size(start{list}));];
end

% find combined segment boundaries
[times, indices] = sort(times);
edges = edges(indices);
edges = cumsum(edges);

% check for overlapping segments
if (max(edges) > lists) | (min(edges) < 0),
  error('one or more segment lists contain overlapping segments');
end

% find AND segments
indices = find(edges == lists);
START = times(indices);
STOP = times(indices + 1);

% create resulting segment list
result = [START STOP];

% remove zero length sequences
result = cullsegments(result);

% merge overlapping or adjacent segments
result = mergesegments(result);

% standardize segment list
result = stdsegments(result);

% return to calling function
return;
