function list = notsegments(list);

%NOTSEGMENTS Determine the complement of a segment list
%
% NOTSEGMENTS returns the segment list corresponding to the complement of
% the given segment list.
%
%SYNOPSIS
%   OUT = notsegments(IN);
%
%INPUT
%   IN        input segment list
%
%OUTPUT
%   OUT       output negated segment list
%
% The segments list should be provided in the form of an N-by-2 matrix,
% formed from adjacent column-wise vectors of segment start and stop
% times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% As an alternative, each segment may also be specified by its start time
% and duration.  However, the resulting segment list will be returned in
% terms of start and stop times.
%
% NOTSEGMENTS also uses CULLSEGMENTS and MERGESEGMENTS to remove zero
% length segments and merge any overlapping or adjacent segments.
%
% NOTSEGMENTS allows for semi-infinite and infinite segments, which may
% extend beyond the end of a data run.  To form the complement of a list
% within the constraints of a data run, use NOTSEGMENTS in conjunction
% with ANDSEGMENTS.  For instance, to create a list of non-playground
% segments within a data run, use the following construction.
%
% playground = playsegments(datarun);
% notplayground = andsegments(notsegments(playground), datarun);
% 
% See also ANDSEGMENTS, ORSEGMENTS, NANDSEGMENTS and NORSEGMENTS.
%
%AUTHOR
%   Shourov K. Chatterji (No longer part of the LC Collaboration)

% $Id: notsegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(1, 1, nargin));

% standardize segment list
list = stdsegments(list);

% remove zero length segments
list = cullsegments(list);

% merge overlapping or adjacent segments
list = mergesegments(list);

% handle empty lists
if isempty(list),
  list = [-Inf +Inf];
  return;
end

% extract start and stop vectors
start = list(:,1);
stop = list(:,2);

% label transitions
times = [start; stop;];
edges = [ones(size(start)); -ones(size(stop));];

% sort transitions
[times, indices] = sort(times);
edges = edges(indices);
edges = cumsum(edges);

% check for overlapping segments
if (max(edges) > 1) | (min(edges) < 0),
  error('one or more segment lists contain overlapping segments');
end

% handle infinite end segments
if times(1) ~= -Inf,
  times = [-Inf; times;];
  edges = [0; edges;];
end
if times(end) ~= +Inf,
  times = [times; +Inf;];
  edges = [edges; 1;];
end

% find merged segment boundaries
indices = find((edges == 0) & (times ~= +Inf));
START = times(indices);
STOP = times(indices+1);

% create return segment list
list = [START STOP];

% standardize segment list
list = stdsegments(list);

% remove zero length segments
list = cullsegments(list);

% merge overlapping or adjacent segments
list = mergesegments(list);

% return to calling function
return;
