function list = padsegments(list, seconds);

%PADSEGMENTS pad/trim extra time on/off segments
%
% PADSEGMENTS appends the requested number of seconds to the beginning
% and ending of each segment in a segment list.  The number of seconds
% may be negative, causing the specified number of seconds to be removed
% from the ends of each segment.  The resulting segment list is returned.
%
%SYNOPSIS
%   OUT = padsegments(IN, SECONDS);
%
%INPUTS
%   IN        input segment list
%   SECONDS   number of seconds to pad/trim
%
%OUTPUT
%   OUT       output segment list
%
% The segments list should be provided in the form of an N-by-2 matrix,
% formed from adjacent column-wise vectors of segment start and stop
% times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% As an alternative, each segment may also be specified by its start time
% and duration.  However, the resulting segment list will be returned in
% terms of start and stop times.
%
% Padding may result in segments extending beyond a data run.  This can
% be corrected by forming the intersection of the resulting list with
% the segment representing the entire run.
%
%   list = padsegment(list, padding);
%   list = andsegments(list, fullrun);
%
% PADSEGMENTS also uses CULLSEGMENTS and MERGESEGMENTS to remove zero
% length segments and merge any overlapping or adjacent segments after
% trimming/padding.
%
% See also CULLSEGMENTS, MERGESEGMENTS, and ALIGNSEGMENTS.
%
%AUTHOR
% Shourov K. Chatterji (No longer part of the LC Collaboration)

% $Id: padsegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(2, 2, nargin));

% standardize segment list
list = stdsegments(list);

% remove zero length segments
list = cullsegments(list);

% merge overlapping or adjacent segments
list = mergesegments(list);

% handle empty lists
if isempty(list),
  return;
end

% extract start and stop vectors
start = list(:,1);
stop = list(:,2);

% pad/trim segments
start = start - seconds;
stop = stop + seconds;

% create return segment list
list = [start stop];

% remove zero length segments
list = cullsegments(list);

% merge overlapping or adjacent segments
list = mergesegments(list);

% return to calling function
return;
