function result = playsegments(varargin);

%PLAYSEGMENTS Determine instersection of lists with playground definition
%
% PLAYSEGMENTS returns the segment list corresponding to the intersection
% of the given segment lists and the playground definition.
%
%SYNOPSIS
%   LIST = playsegments(LIST1, LIST2, ...);
%
% For instance, the following script writes generates the list of
% playground segments for the second LIGO science run.
%
%   run = [729273613 734371213];
%   playground = playsegments(run);
% 
% By definition, the playground consists of 600 second segments, whose
% start times are non-negative integer multiples of 6370 seconds from the
% beginning of the second LIGO science run (7292736313).  The formal
% defintion of the playground is contained in the LIGO internal document
% LIGO-T030020-01-Z.  This playground definition does not extend prior to
% the start of the second LIGO science run.  However, the specifically
% designated playground list for the first LIGO science run also taken into
% account.
% 
%INPUTS
%   LIST1, LIST2, ... - Segment lists in the form of N-by-2 matrices,
%     formed from adjacent column-wise vectors of segment start and stop
%     times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% As an alternative, each segment may also be specified by its start time
% and duration.  However, the resulting segment list will be returned in
% terms of start and stop times.
%
%OUTPUT
%   LIST - intersection of playground segments
%
% PLAYSEGMENTS also uses ANDSEGMENTS, CULLSEGMENTS, and MERGESEGMENTS to
% combine lists, remove zero length segments, and merge any overlapping or
% adjacent segments.
%
% See also ANDSEGMENTS, NANDSEGMENTS, ORSEGMENTS, and NORSEGMENTS.
%
%AUTHOR
%   Shourov K. Chatterji (No longer part of the LC Collaboration)

% $Id: playsegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(1, Inf, nargin));

% determine intersection of the given lists
list = andsegments(varargin{:});

% handle empty lists
if isempty(list),
  result = [];
  return;
end

% playground definition
playground_start      = 729273613;
playground_separation = 6370;
playground_duration   = 600;

% determine time range for given lists
list_start = min(list(:, 1));
list_stop = max(list(:, 2));

% index of first playground segment in possible coincidence
first_index = ceil((list_start - playground_duration - playground_start) / ...
                    playground_separation + eps);

% index of last playground segment in possible coincidence
last_index = floor((list_stop - playground_start) / ...
                    playground_separation - eps);

% force valid indices for playground definition
first_index = max(first_index, 0);

% vector of indices for playground segments in possible coincidence
indices = first_index : last_index;

% playground segments start and stop times
segments_start = playground_start + playground_separation * indices;
segments_stop  = segments_start + playground_duration;

% create playground segment list
playground = [714265025 714266239; ...
              714357173 714357806; ...
              714382917 714388358; ...
              714462428 714464278; ...
              714559457 714560204; ...
              714636028 714640435; ...
              714821115 714823144; ...
              714941207 714944756; ...
              715042855 715049751; ...
              715087222 715087871; ...
              715219955 715224063; ...
              715344556 715345510; ...
              715482685 715483545; ...
              segments_start(:) segments_stop(:)];

% determine intersection of the given lists and the playground definition
result = andsegments(playground, list);

% remove zero length sequences
result = cullsegments(result);

% merge overlapping or adjacent segments
result = mergesegments(result);

% return to calling function
return;
